##-*- Mode: CPerl -*-

## File: Wlanctl
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##  + perl interface to linux iwtools
##======================================================================

package Wlanctl;
use Wlanctl::Config;
use Linux::Iwtools qw(:all);
use Exporter;
use IO::File;
use Carp;
use Data::Dumper;
use strict;

our @ISA = qw(Exporter);
our $VERSION = 0.03;

our %EXPORT_TAGS =
  (
   utils=>[qw(sysrun)],
   verbose=>[qw(%vlevels)],
  );
our @EXPORT_OK = map { @$_ } values(%EXPORT_TAGS);
our @EXPORT    = qw();

$EXPORT_TAGS{all} = \@EXPORT_OK;

##======================================================================
## Configuration Files
##======================================================================

our @RCFILES = ("${Wlanctl::Config::DEFAULT_CONFIG_DIR}/wlanctl",
		"$ENV{HOME}/.wlanctlrc");
#our @RCFILES = ("wlanctlrc");

##======================================================================
## Verbosity Levels
##======================================================================

our %vlevels = (
		default=>4,
		##
		silent=>0,
		error=>1,
		warning=>2,
		info=>3,
		trace=>4,
		debug=>5,
		(map { ("debug$_"=>($_+5)) } (1..10)),
	       );

##======================================================================
## Signals
##======================================================================
#use Config;
#our @SIGNAME = split(/\s+/, $Config{sig_name});
#our %SIGNUM  = map { $SIGNAME[$_]=>$_ } (0..$#SIGNAME);

##======================================================================
## Object Defaults
##======================================================================
our %DEFAULTS =
  (
   ##-- general options
   verbose=>'info',        	##-- verbosity level

   ##-- device options
   device=>'eth1',		##-- wireless device
   module=>'eth1',		##-- kernel module (undef for none)
   pcmcia=>1,			##-- is this a pcmcia card?

   ##-- default iwconfig configuration
   dhcp => 1,			##-- whether to do dhcp by default
   iwparams=> {
	       key=>'off',
	       essid=>'any',
	       mode=>'managed',
	      },

   ##-- on/off hooks
   pre_enable_hooks=>undef,
   post_enable_hooks=>undef,
   pre_disable_hooks=>undef,
   post_disable_hooks=>undef,

   ##-- new networks
   newctr=>0,

   ##-- networks, sorted by priority
   ##   + Known Keys:
   ##      name => $symbolic_name,
   ##      iwparams => \%args_for_iwconfig,
   ##      dhcp => $bool,
   networks =>
   [
    {
     name=>'auto',
     dhcp=>1,
     pre_up_hooks=>undef,
     post_up_hooks=>undef,
     pre_down_hooks=>undef,
     post_down_hooks=>undef,
     iwparams=>{
		key=>'off',
		essid=>'any',
		mode=>'managed',
	       },
    },
   ],

   ##-- association options
   wait_tic=>2, 	        ##-- secs to wait between assocation checks
   wait_max=>10,		##-- max secs to wait for association

   ##-- state file
   statefile => '/var/wlanctl/state',

   ##-- programs
   iwconfig => $Wlanctl::Config::IWCONFIG, ##-- used for set, so sudo is happy
   iwlist   => $Wlanctl::Config::IWLIST,   ##-- obsolete
   sudo     => $Wlanctl::Config::SUDO,
   modprobe => $Wlanctl::Config::MODPROBE,
   cardctl  => $Wlanctl::Config::CARDCTL,

   ifconfig => $Wlanctl::Config::IFCONFIG,
   route    => $Wlanctl::Config::ROUTE,

   dhclient => $Wlanctl::Config::DHCLIENT,
   dhclient_stop => $Wlanctl::Config::DHCLIENT_STOP,
  );

##======================================================================
## Constructor
##======================================================================

## $wd = Wlanctl->new(%args)
##  + %args:
##    device => $device, ##-- default: eth1
##    ...
sub new {
  my $that = shift;
  my $wd = bless({%DEFAULTS,@_}, ref($that)||$that);
  $wd->{default_network} = $wd if (!$wd->{default_network});
  return $wd;
}

## undef = $wd->DESTROY()
sub DESTROY {
  my $wd = shift;
  delete($wd->{default_network});
  $wd->_unindex;
}

##======================================================================
## Messages
##======================================================================

## undef = $wd->vmsg($level,@message)
sub vmsg {
  my $wd = shift;
  my $level = shift;
  $wd->{verbose} = 'default' if (!defined($wd->{verbose}));
  if ($vlevels{$level} <= $vlevels{$wd->{verbose}}) {
    print STDERR (ref($wd)||$wd, "[$wd->{device}]: ", @_);
  }
}

## undef = $wd->vmsgnp($level,@message)
##  + no prefix
sub vmsgnp {
  my $wd = shift;
  my $level = shift;
  $wd->{verbose} = 'default' if (!defined($wd->{verbose}));
  if ($vlevels{$level} <= $vlevels{$wd->{verbose}}) {
    print STDERR @_;
  }
}

## $rc = $wd->sysrun(@cmd_and_args)
##  + really just a wrapper for 'system()'
sub sysrun {
  my ($wd,@cmd) = @_;
  $wd->{verbose} = 'default' if (!defined($wd->{verbose}));

  if ($vlevels{debug} <= $vlevels{$wd->{verbose}}) {
    ##-- full debug message
    $wd->vmsg('debug', "sysrun(", join(' ', map { "'$_'" } @cmd), ")\n");
  }
  elsif ($vlevels{trace} <= $vlevels{$wd->{verbose}}) {
    ##-- trace message
    $wd->vmsg('trace', "sysrun($cmd[0])\n");
  }

  unshift(@cmd,$wd->{sudo}) if ($wd->{sudo});
  return system(@cmd);
}

##======================================================================
## Device Discovery
##======================================================================

## @devs = $CLASS_OR_OBJ->devices()
## @devs = $CLASS_OR_OBJ->devices($file)
## @devs = $CLASS_OR_OBJ->devices($file,$regex)
##  + $file defaults to Linux::Iwtools::PROC_NET_WIRELESS() [i.e. '/proc/net/wireless']
sub devices {
  my ($wd,$file,$regex) = @_;
  $file = PROC_NET_WIRELESS() if (!defined($file));
  $regex = '^\s*([^:]*):' if (!defined($regex));

  my $wls = IO::File->new("<$file");
  if (!$wls) {
    carp(ref($wd)||$wd, "::iwdevices() could not open '$file': $!");
    return qw();
  }
  my @devs = qw();
  while (<$wls>) {
    if (/^\s*([^:]*):/) {
      push(@devs, $1);
    }
  }
  $wls->close;

  return @devs;
}

##======================================================================
## System Wrappers
##======================================================================

## $rc = sys_iwconfig %args
##  + just wraps a system call
sub sys_iwconfig {
  my $wd = shift;
  $wd->sysrun($wd->{iwconfig},$wd->{device},@_);
}
##--> NOW IMPORTED from Linux::Iwtools

## $rc = $wd->iwlist @args
sub sys_iwlist {
  my $wd = shift;
  $wd->sysrun($wd->{iwlist},$wd->{device},@_);
}
##--> NOW IMPORTED from Linux::Iwtools

##======================================================================
## Get Status
##======================================================================

## \%status = $wd->getStatus();
##   + gets status, defaults to $wd->{status}
sub status {
  my $wd = shift;
  return $wd->{status} && !$wd->{status}{stale} ? $wd->{status} : $wd->getStatus;
}

## \%status = $wd->{status} = $wd->getStatus()
sub getStatus {
  my $wd = shift;
  my $status = $wd->{status};
  if (!$status) { $status = $wd->{status} = {};  }

  my $net = $wd->{current_network};
  my $if  = $status->{if} = `$wd->{ifconfig} $wd->{device} 2>&1`;
  my $iw  = $status->{iw} = `$wd->{sudo} $wd->{iwconfig} $wd->{device} 2>&1`;
  #my $iw  = $status->{iw} = iwconfig($wd->{device})->{$wd->{device}};

  my $cc  = $status->{cc} = `$wd->{cardctl}  status 2>&1`;
  my @rts = map { [split] } grep { chomp; /$wd->{device}$/ } `$wd->{route} -n`;

  $status->{name}    = ($net && $net->{name} ? $net->{name} : '');

  $status->{ison}    = 1;
  $status->{ison}    = 0 if ($wd->{pcmcia} && $cc !~ /\[ready\]/m);
  $status->{ison}    = 0 if ($iw =~ /No such device/);
  #$status->{ison}    = 0 if (!ref($iw));

  $status->{isup}    = ($status->{ison} && $if =~ /\bUP\b/);
  $status->{isassoc} = ($status->{isup}
			#&& defined($iw->{ap})
			#&& $iw->{ap} ne '00:00:00:00:00:00'
			&& $iw !~ /00:00:00:00:00:00/
			&& $if =~ /\binet addr\b/
		       );

  $status->{isencrypted} = ($status->{ison}
			    && $iw =~ /\bEncryption key:/
			    && $iw !~ /\bEncryption key:\s*off\b/
			    #&& defined($iw->{key})
			    #&& $iw->{key} ne 'off'
			   );


  #$status->{essid}  = defined($iw->{essid}) ? $iw->{essid} : '';
  #$status->{ap}     = defined($iw->{ap})    ? $iw->{ap}    : '';
  #$status->{link}   = $iw->{link_qual};
  ##--
  $status->{essid}  = ($iw =~ /\s+ESSID:\s*\"([^\"]*)\"/ ? $1 : '');
  $status->{ap}     = ($iw =~ / Access Point:\s*([0-9A-Fa-f:]*)/m ? $1 : '');
  $status->{link}   = ($iw =~ /^\s+Link Quality=\s*(\d+)/m ? $1 : '');
  $status->{ipaddr} = ($if =~ /^\s+inet addr:([0-9\.]+)/m ? $1 : '');

  $status->{ipaddr} = ($if =~ /^\s+inet addr:([0-9\.]+)/m ? $1 : '');
  $status->{routes} = \@rts;

  $status->{stale} = 0;

  return $status;
}


## $txtStatus = $wd->statusText()
## $txtStatus = $wd->statusText(\%status)
##  + instantiates all keys in \%status
sub statusText {
  my $wd = shift;
  my $status = shift;
  $status = $wd->status if (!$status);

  my %ts = %$status;
  $ts{name} = '?' if (!$ts{name});
  $ts{ison} = ($ts{ison} ? 'ON' : 'OFF');
  $ts{isup} = ($ts{isup} ? 'UP' : 'DOWN');
  $ts{isassoc} = ($ts{isassoc} ? 'ASSOCIATED' : 'UNASSOCIATED');
  $ts{isencrypted} = ($ts{isencrypted} ? 'ENABLED' : 'OFF');
  $ts{essid} = '?' if (!$ts{essid});
  $ts{ap} = '00:00:00:00:00:00' if (!$ts{ap});
  $ts{link}  = 0   if (!$ts{link});
  $ts{ipaddr} = '?' if (!$ts{ipaddr});
  my ($dst,$gw,$mask);
  $ts{routes} = [
		 #map { [m/^([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)/] }
		 map { [@$_[0,1,2]] }
		 @{$status->{routes}}
		];

  return \%ts;
}

##======================================================================
## Enable/Disable
##======================================================================

## $bool = $wd->enable()
*on = \&enable;
sub enable {
  my $wd = shift;
  #my $status = $wd->{status};

  $wd->runHooks($wd->{pre_enable_hooks});
  $wd->vmsg('info', "enable()\n");

  if ($wd->{pcmcia} && `$wd->{cardctl} status` !~ /ready/) {
    $wd->sysrun($wd->{cardctl}, 'insert')==0 or return 0;
  }
  if ($wd->{module}) {
    $wd->sysrun($wd->{modprobe}, $wd->{module})==0 or return 0;
  }

  ##-- set to promiscuous mode by default
  #iwconfig($wd->{device},%{$wd->{iwparams}})->{status}==0 or return 0;
  $wd->sys_iwconfig(%{$wd->{iwparams}})==0 or return 0;

  ##-- this shouldn't happen here, since it messes
  ##   up dhclient's route handling
  #$wd->sysrun($wd->{ifconfig}, $wd->{device}, 'up');

  $wd->{status}{stale} = 1; ##-- mark status as stale
  $wd->runHooks($wd->{post_enable_hooks});

  return 1;
}

## $bool = $wd->wlan_disable()
*off = \&disable;
sub disable {
  my $wd = shift;

  $wd->runHooks($wd->{pre_disable_hooks});
  $wd->vmsg('info', "disable()\n");

  $wd->sysrun($wd->{ifconfig}, $wd->{device}, 'down');
  if ($wd->{module}) {
    $wd->sysrun($wd->{modprobe}, '-r', $wd->{module})==0 or return 0;
  }
  if ($wd->{pcmcia} && `$wd->{cardctl} status` =~ /ready/) {
    $wd->sysrun($wd->{cardctl}, 'eject')==0 or return 0;
  }

  $wd->{status}{stale} = 1; ##-- mark status as stale
  $wd->runHooks($wd->{post_disable_hooks});

  return 1;
}


##======================================================================
## ifup / ifdown
##======================================================================

## $bool = $wd->ifup()
sub ifup {
  my $wd = shift;
  $wd->vmsg('info', "ifup()\n");
  $wd->sysrun($wd->{ifconfig}, $wd->{device}, 'up');
  $wd->{status}{stale} = 1; ##-- mark status as stale
  return 1;
}

## $bool = $wd->ifdown()
sub ifdown {
  my $wd = shift;
  $wd->vmsg('info', "ifup()\n");
  $wd->sysrun($wd->{ifconfig}, $wd->{device}, 'down');
  $wd->{status}{stale} = 1; ##-- mark status as stale
  return 1;
}


##======================================================================
## Scanning
##======================================================================

## @scanned = $wd->scan()
##  + @scanned = (\%net1, ...)
##  + %net  = (
##      cell  => $cellid,
##      ap => $ap,
##      mode  => $mode,
##      encryption => $bool,
##      channel => $channel,
##      quality => $quality,
##      signal => $level,
##      noise => $level,
##     )
*iwscan = \&scan;
sub scan {
  my $wd = shift;
  my $status = $wd->status;
  my $scandev = $wd->{device};

  ##-- must run device up for scan
  if (!$status->{isup}) {
    $wd->ifup;
    sleep(1);
  }

  my $results = iwlist($scandev,'scanning');
  return qw() if ($results->{status}!=0);

  return @{$results->{$scandev}};
}

##======================================================================
## Association
##======================================================================

## $bool = $wd->waitForAssociation()
*wait = *waitForAssoc = *waitForAssociation;
sub waitForAssociation {
  my $wd = shift;
  my $waited = 0;
  my ($iw);
  $wd->vmsg('info', "waitForAssoc ($wd->{wait_tic}<=$wd->{wait_max}) ");
  while ($waited < $wd->{wait_max}) {
    $wd->vmsgnp('info','.');
    sleep($wd->{wait_tic});
    $waited += $wd->{wait_tic};

    #if (`$wd->{iwconfig} $wd->{device}` !~ /00:00:00:00:00:00/) {
    $iw = iwconfig($wd->{device});
    if ($iw->{status}==0
	&& ref($iw->{$wd->{device}})
	&& defined($iw->{$wd->{device}}{ap})
	&& $iw->{$wd->{device}}{ap} ne '00:00:00:00:00:00')
      {
	$wd->vmsgnp('info',"\n");
	$wd->vmsg('info', "ASSOCIATED after $waited seconds.\n");
	return 1;
      }
  }
  $wd->vmsgnp('info', "\n");
  $wd->vmsg('info', "NOT associated in $wd->{wait_max} seconds.\n");
  return 0;
}

##======================================================================
## Up/Down
##======================================================================

## $bool = $wd->connect(\%network)
##  + backwards-compatible alias: up()
##  + runs up the wlan
##  + %network:
##      iwparams =>\%args_for_iwconfig,
##      dhcp     => $bool,               ##-- default=1
*up = \&connect;
sub connect {
  my ($wd,$net) = @_;
  $net = $wd->{default_network} if (!$net);

  $wd->runHooks($net->{pre_up_hooks});
  $wd->vmsg('info', "up(", ($net->{name} ? $net->{name} : ''), ")\n");

  if ($net->{iwparams}) {
    #iwconfig($wd->{device}, %{$net->{iwparams}})->{status}==0
    $wd->sys_iwconfig(%{$net->{iwparams}})==0
      or return 0;
  }

  $wd->sysrun($wd->{ifconfig}, $wd->{device}, 'up');
  $wd->wait() or return 0;

  if ($net->{dhcp}) {
    $wd->sysrun($wd->{dhclient},
		'-pf',
		"/var/run/dhclient.$wd->{device}.pid", $wd->{device})==0
		  or return 0;
  }

  $wd->{status}{stale} = 1; ##-- mark status as stale

  ##-- save current network & relevant state
  $wd->{current_network} = $net;
  $wd->saveState();
  $wd->runHooks($net->{post_up_hooks});

  return 1;
}


## $bool = $wd->disconnect(\%network)
##  + backwards-compatible alias: down()
##  + runs down the wlan
##  + %network:
##      dhcp     => $bool,               ##-- default=1
*down = \&disconnect;
sub disconnect {
  my ($wd,$net) = @_;
  $net = $wd->{current_network} if (!$net);
  $net = $wd->{default_network} if (!$net);

  $wd->runHooks($net->{pre_down_hooks});
  $wd->vmsg('info', "down(", ($net->{name} ? $net->{name} : ''), ")\n");

  if ($net->{dhcp}) {
    $wd->vmsg('info', "terminating DHCP client\n");
    $wd->sysrun($wd->{dhclient_stop}, $wd->{device});
  }

  delete($wd->{current_network});
  $wd->saveState();

  ##-- re-instantiate default network
  #iwconfig($wd->{device}, %{$wd->{iwparams}});
  $wd->sys_iwconfig(%{$wd->{iwparams}});

  $wd->{status}{stale} = 1; ##-- mark status as stale
  $wd->runHooks($net->{post_down_hooks});
  return 1;
}

##======================================================================
## Configuration files
##======================================================================

## $wd = $wd->readConfigFile($file)
##  + reads in a config file
##  + config file (perl program text) may use '$_' to refer to the object
*readConfig = \&readConfigFile;
sub readConfigFile {
  my ($wd,$file) = @_;

  $wd->vmsg('trace', "readConfigFile($file)\n");

  local $_ = $wd;
  do "$file";
  carp(ref($wd), ": Error during readConfigFile($file): $@") if ($@);
  return $wd;
}

## $wd = $wd->readDefaultConfig()
##  + reads in default config files:
##    /etc/default/wlanctl
##    $HOME/.wlanctlrc
sub readDefaultConfig {
  my $wd = shift;
  my ($rcfile);
  foreach $rcfile (@RCFILES) {
    $wd->readConfigFile($rcfile) if (-r $rcfile);
  }
  return $wd;
}

##======================================================================
## State File
##======================================================================

## $bool = $wd->saveState()
##  + saves state file
sub saveState {
  my $wd = shift;
  my $file = $wd->{statefile};
  my $fh = IO::File->new(">$file");
  if (!$fh) {
    carp(ref($wd), "::saveState(): open failed for '$file': $!");
    return 0;
  }
  $fh->print(Data::Dumper->Dump([
				 {
				  device=>$wd->{device},
				  current_network=>$wd->{current_network},
				 },
				],
				['state']));
  $fh->close();
  return 1;
}

## $bool = $wd->loadState()
sub loadState {
  my $wd = shift;
  my $file = $wd->{statefile};
  my $state = do $file;
  carp(ref($wd), ": Error during loadState($file): $@") if ($@);
  @$wd{keys %$state} = values %$state;
  return 1;
}

## $bool = $wd->clearState()
sub clearState {
  my $wd = shift;
  my $fh = IO::File->new(">$wd->{statefile}");
  if (!$fh) {
    carp(ref($wd), "::clearState(): could not truncate '$wd->{statefile}': $!");
    return 0;
  }
  $fh->close();
  return 1;
}

## undef = $wd->showState();
*State = *showStatus = \&showState;
sub showState {
  my $wd = shift;
  my $status = $wd->statusText;

  my ($dst,$gw,$mask);
  print
    (
     "$wd->{device} $status->{ison} $status->{isup} $status->{isassoc}\n",
     "  Name   : $status->{name}\n",
     "  ESSID  : $status->{essid}\n",
     "  WEP    : $status->{isencrypted}\n",
     "  AP     : $status->{ap}\n",
     "  Link Q : $status->{link}\n",
     "  IP     : $status->{ipaddr}\n",
     "  Routes : ",
     join("\n".(' ' x 11),
	  (map {
	    ($dst,$gw,$mask)=@$_;
	    sprintf("%-15s / %-15s  gw=%-15s", $dst, $mask, $gw)
	  } @{$status->{routes}})),
     "\n",
    );
}

##======================================================================
## Network Indexing
##======================================================================

## $wd = $wd->_index()
##  + indexes stored networks
sub _index {
  my $wd = shift;

  $wd->{networks_byap} = {};
  $wd->{networks_byessid} = {};
  $wd->{networks_byname}  = {};

  my ($net,$essid,$name,$ap, $idx);
  my $saw_default = 0;
  $idx = 0;
  foreach $net (@{$wd->{networks}}) {
    $net->{index} = $idx++;
    if ($net->{default} && !$saw_default) {
      $wd->{default_network} = $net;
      $saw_default = 1;
    } else {
      $net->{default} = 0;
    }

    ##-- normalize ap macs to lower-case
    if ($ap=$net->{ap}) {
      $ap =~ tr/A-F/a-f/;
      $wd->{networks_byap}{$ap}  = $net;
      $net->{ap} = $ap;
    }

    $wd->{networks_byessid}{$net->{iwparams}{essid}} = $net
      if ($net->{iwparams}{essid}
	  && $net->{iwparams}{essid} ne 'any'
	  && $net->{iwparams}{essid} ne 'off');

    $wd->{networks_byname}{$net->{name}} = $net
      if ($net->{name});
  }

  ##-- final default attempt
  if (!$saw_default && @{$wd->{networks}}) {
    $net = $wd->{default_network} = $wd->{networks}[0];
    $net->{default} = 1;
  }

  return $wd;
}

## $wd = $wd->_unindex()
##  + clears indices (for DESTROY)
sub _unindex {
  my $wd = shift;
  %$_ = qw() foreach (@$wd{qw(networks_byap networks_byname networks_byessid)});
}

##======================================================================
## Network Matching
##======================================================================


## \%net = $wd->match(\%scanned_network);
##   + returns full network hash, creating one if necessary
sub match {
  my ($wd,$scanned) = @_;
  $wd->_index if (!defined($wd->{networks_byap}));

  my ($net,$ap,$essid);
  if ($ap=$scanned->{ap}) {
    ##-- match by AP mac
    $ap =~ tr/A-F/a-f/;
    $net = $wd->{networks_byap}{$ap};
  }

  ##-- match by essid
  $net = $wd->{networks_byessid}{$essid} if (!$net && ($essid=$scanned->{essid}));

  ##-- no match: construct new network
  if (!$net) {
    $net = {
	    name => ("unknown".(++$wd->{newctr})),
	    ap=>$ap,
	    dhcp=>$wd->{dhcp},
	    index=>scalar(@{$wd->{networks}}),
	    iwparams=> { %{$wd->{iwparams}} },
	   };
    push(@{$wd->{networks}}, $net);
    $wd->{networks_byap}{$ap} = $net;
    $wd->{networks_byessid}{$essid} = $net if ($essid);
    $wd->{networks_byname}{$net->{name}} = $net;
  }

  ##-- store scan results with network
  $net->{scan} = $scanned;

  return $net;
}


##======================================================================
## List known networks
##======================================================================

## undef = $wd->listNetworks
sub listNetworks {
  my $wd = shift;

  my $statlen = 1;
  my $namelen = 14;
  my $essidlen = 32;
  my $maclen = 17;
  my $weplen = 5;
  my $dhcplen = 5;

  ##-- header
  printf("%${statlen}s %-${namelen}s %-${essidlen}s %-${maclen}s %${weplen}s %-${dhcplen}s\n"
	 .("-" x 79)."\n",
	 qw(S NAME ESSID AP-MAC WEP? DHCP?));

  my ($net,$name,$essid,$ap,$wep,$dhcp,$iwparams);

  my $default_net = $wd->{default_network};
  $default_net = '' if (!$default_net);

  my $current_net = $wd->{current_network};
  $current_net = '' if (!$current_net);

  foreach $net (@{$wd->{networks}}) {
    $iwparams = $net->{iwparams};
    printf("%${statlen}s %-${namelen}s %-${essidlen}s %-${maclen}s %${weplen}s %${dhcplen}s\n",
	   ($net eq $default_net
	    ? '*'
	    : ($net eq $current_net
	       ? 'C'
	       : '')),
	   ($net->{name} ? $net->{name} : '(undef)'),
	   ($iwparams->{essid} ? $iwparams->{essid} : 'off'),
	   ($net->{ap} ? $net->{ap} : 'off'),
	   ($iwparams->{key} && $iwparams->{key} ne 'off' ? 'yes' : 'no'),
	   ($net->{dhcp} ? 'yes' : 'no'),
	 );

  }
}


##======================================================================
## List scan results
##======================================================================

## @matches = $wd->scanMatch()
sub scanMatch {
  my $wd = shift;
  return
    (sort
     { $a->{index} <=> $b->{index} }
     (map { $wd->match($_) } $wd->scan));
}

## undef = $wd->scanVerbose()
##  + calls scan() and listScanResults()
*Scan = \&scanVerbose;
sub scanVerbose {
  my $wd = shift;
  $wd->listScanResults($wd->scanMatch);
}


## undef = $wd->listScanResults(@scanned_and_matched)
sub listScanResults {
  my ($wd,@scanned) = @_;

  my $namelen = 16;
  my $essidlen = 29;
  my $maclen = 18;
  my $weplen = 4;
  my $chanlen = 3;
  my $quallen = 4;

  ##-- header
  printf("%-${namelen}s %-${essidlen}s %-${maclen}s %-${quallen}s %-${weplen}s %-${chanlen}s\n"
	 .("-" x 79)."\n",
	 qw(NAME ESSID AP-MAC LNK WEP CHN));

  my ($scan, $net,$name,$essid,$ap,$level,$wep,$chan,$iwparams);

  foreach $net (@scanned) {
    $scan = $net->{scan};
    $iwparams = $net->{iwparams};

    printf("%-${namelen}s %-${essidlen}s %-${maclen}s %-${quallen}d %-${weplen}s %${chanlen}d\n",
	   ($net && $net->{name} ? $net->{name} : '(unknown)'),
	   ($iwparams->{essid} ? $iwparams->{essid} : '(unknown)'),
	   $scan->{ap},
	   $scan->{link_qual},
	   ($scan->{key} && $scan->{key} ne 'off' ? 'yes' : 'no'),
	   $scan->{channel},
	  );
  }
  if (!@scanned) {
    print "(no available networks found)\n";
  }
}


##======================================================================
## User hooks
##======================================================================

## undef = $wd->runHooks(@hooks)
## undef = $wd->runHooks(\@user_hooks)
##  + each element of @hooks is either:
##    - a string, which is run via system() (no implicit sudo here!), OR
##    - a subroutine ref, evaluated with argument $wd
sub runHooks {
  my $wd = shift;

  my @hooks = map { ref($_) && ref($_) eq 'ARRAY' ? @$_ : $_ } @_;
  foreach (@hooks) {
    next if (!defined($_));
    $wd->vmsg('trace', "runHook($_)\n");
    if (ref($_) && ref($_) eq 'CODE') {
      $_->($wd);
    }
    elsif (!ref($_)) {
      if (system($_)!=0) {
	$wd->vmsg('warn', "System hook returned non-zero: '$_'\n");
      }
    }
    else  {
      $wd->vmsg('error', "::runHooks(): unknown hook type ", ref($_), "\n");
    }
  }
}


1;

##======================================================================
## Docs
=pod

=head1 NAME

Wlanctl - wireless control utilities

=head1 SYNOPSIS

 use Wlanctl;

 #...

=cut

##======================================================================
## Description
=pod

=head1 DESCRIPTION

Not yet written.

=cut

##=====================================================================
## Footer
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2004, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1),

=cut
