#!/usr/bin/perl -w

##----------------------------------------------------------------------
## File: wlanctl.perl
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##   + command-line control interface for Wlanctl.pm module
##----------------------------------------------------------------------

use lib qw(.);
use Wlanctl qw(:verbose);
use Getopt::Long;
use Pod::Usage;
use File::Basename qw(basename);
##----------------------------------------------------------------------
## Globals
##----------------------------------------------------------------------
our $VERSION = $Wlanctl::VERSION;
our $PROG    = basename($0, '.perl');

##----------------------------------------------------------------------
## Defaults
##----------------------------------------------------------------------
our %config =
  (
   verbose=>'default',
  );
our $network_name = undef;

##----------------------------------------------------------------------
## Globals
##----------------------------------------------------------------------
GetOptions(##-- general options
	   'help|h!' => \$help,
	   'man!'    => \$man,
	   'version!' => \$version,
	   'nobanner|nb'  => \$nobanner,

	   ##-- configuration options
	   'config|c=s'  => \$alt_config_file,
	   'verbose|v:s'  => \$config{verbose},
	   'netname|net|name|n=s' => \$network_name,
	  );

pod2usage({
	   -exitval=>0,
	   -verbose=>0
	  }) if ($help);
pod2usage({
	   -exitval=>0,
	   -verbose=>1
	  }) if ($man);

if (!$nobanner && ($version || $vlevels{$config{verbose}} >= $vlevels{info})) {
  print "$PROG version $VERSION by Bryan Jurish\n";
  exit 0 if ($version);
}

##======================================================================
## MAIN
##======================================================================

##-- create object
$wd = Wlanctl->new(%config);

##-- read config, state
$wd->readDefaultConfig;
$wd->readConfigFile($alt_config_file) if ($alt_config_file);
$wd->loadState;
$wd->_index();

$wd->{default_network} = $wd->{networks_byname}{$network_name}
  if ($network_name);

##-- perform actions
foreach $act (@ARGV) {

  ##----------------------------------------------------
  ## On/off
  if ($act eq 'on' || $act eq 'prep') {
    die("$PROG: action 'on' FAILED!\n") if (!$wd->on);
  }
  elsif ($act eq 'off' || $act eq 'unprep') {
    die("$PROG: action 'off' FAILED!\n") if (!$wd->off);
  }

  ##----------------------------------------------------
  ## ifup/ifdown
  elsif ($act eq 'ifup') {
    die("$PROG: action 'ifup' FAILED!\n") if (!($wd->on && $wd->ifup));
  }
  elsif ($act eq 'ifdown') {
    die("$PROG: action 'ifdown' FAILED!\n") if (!($wd->down && $wd->ifdown));
  }

  ##----------------------------------------------------
  ## Up/Down
  elsif ($act eq 'start' || $act eq 'up' || $act eq 'attach') {
    die("$PROG: action 'up' FAILED!\n") if (!($wd->on && $wd->up));
  }
  elsif ($act eq 'stop' || $act eq 'down' || $act eq 'detach') {
    #die("$PROG: action 'down' FAILED!\n") if (!($wd->down && $wd->off));
    die("$PROG: action 'down' FAILED!\n") if (!$wd->down);
  }


  ##----------------------------------------------------
  elsif ($act =~ /^list/ || $act =~ /^nets/) {
    $wd->listNetworks;
  }

  ##----------------------------------------------------
  elsif ($act eq 'scan' || $act =~ /^avail/) {
    $wd->scanVerbose;
  }

  ##----------------------------------------------------
  elsif ($act =~ /^stat/) {
    $wd->showStatus($wd->getStatus);
  }


  ##----------------------------------------------------
  else {
    print STDERR "$PROG: unknown action '$act' -- ignoring.\n";
  }
}

exit 0;

##======================================================================
## Pods
##======================================================================
=pod

=head1 NAME

wlanctl.perl - command-line control for Wlanctl module

=head1 SYNOPSIS

 wlanctl.perl [OPTIONS] ACTION(s)

 General Options:
   -help
   -version
   -nobanner
   -verbose LEVEL

 Configuration Options:
   -netname NETWORK

 Actions:
   on   , off     ##-- enable/disable WLAN device
   ifup , ifdown  ##-- run the device up/down
   up   , down    ##-- connect/disconnect from network
   list           ##-- list known networks
   scan           ##-- scan for available networks
   status         ##-- show current status

=cut

###############################################################
# Arguments
###############################################################
=pod

=head1 ARGUMENTS

ACTION is one of the following:

=over 4


=item on

Enable the WLAN device.

=item off

Disable the WLAN device.


=item ifup

Just run the interface up, do not attempt to connect.

=item ifdown

Just run the interface down.


=item up

Initiate a WLAN connection to the default network.

=item down

Terminate the most recently initiated WLAN connection.



=item list

List known networks.



=item scan

Scan for available networks.



=item status

Show WLAN connection status.



=back

=cut

###############################################################
# Options
###############################################################
=pod

=head1 OPTIONS

=cut

###############################################################
# General Options
###############################################################
=pod

=head2 General Options

=over 4

=item -help

Display a brief help message and exit.

=item -version

Display version information and exit.

=item -verbose LEVEL

Set verbosity level to LEVEL.  Default=1, Max=5.

=item -nobanner

Suppress version banner.

=back

=cut


###############################################################
# Socket Options
###############################################################
=pod

=head2 Configuration Options

Not yet written.

=cut

###############################################################
# Bugs and Limitations
###############################################################
=pod

=head1 BUGS AND LIMITATIONS

Many.

=cut

###############################################################
# Footer
###############################################################
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>moocow@ling.uni-potsdam.deE<gt>

=head1 SEE ALSO

perl(1),
Wlanctl(3pm).

=cut
